'-----------------------------------------------------------------------------------------
' Software bases on:
' Burkhard Kainka
' morse.bas
' morse code decoder with Mega169 and LCD butterfly driver
' using butterfly joystick as morse key


' screwed up by Ralf Beesner, DK5BU

' shrinked to fit into AtTiny13 and changed for ASCII output by software UART
' some keycodes have been added, e.g. + - ? ( ) . ,
' and, most important keycode: space between words ;-)
' all timing constants are derived from the duration of a short morse element ("dit"), so its easier
' to change speed

' there are two ways to insert spaces: by simply waiting (look at mainloop for variable "iconst")
' or by a dedicated character (morsecode <AS>; wait a second) which is better for talking to a computer

' Return (ASCII 13) is generated by the morsecode <KN>
' Backspace (ASCII 8) is generated by morsecode Error (...... and .......).
' Only 6 or 7 dots are recognized as error character (maximum possible length.of a character is 7)

' Buzzer is attached to PB0
' Keyer is attached to PB2
' Speed switch is attached to PB3 (60 or 90 characters per minute)
' Space switch is attached to PB4 (inserts space by idling time or only by character <AS>

'-----------------------------------------------------------------------------------------

$regfile = "Attiny13.dat"                                   ' specify the used micro
$crystal = 1200000                                          ' used clock frequency
$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 12                                               ' default use 10 for the SW stack
$framesize = 0                                              ' default use 40 for the frame space



Dim Key As Byte
Dim C As Byte
Dim Timepressed As Word
Dim Timenotpressed As Word
Dim Charactercode As Byte
Dim Idling As Word
Dim Speed As Byte


Dim Dithalf As Word
Dim Tpconst As Word
Dim Tnpconst As Word
Dim Iconst As Word

Declare Sub Getkey
Declare Sub Getkeytimes
Declare Sub Initcode
Declare Sub Printdisplay



Portb = &B00011100                                          ' pullups for PB2,PB3,PB4
Ddrb = &B00000010                                           ' PB1 is UART output, all other pins are inputs
Didr0 = &B0000011                                           ' inputs PB0 and PB1 are deactivated

' Timer 0 is configured for Sound at OC00A / PB0

Config Timer0 = Timer , Prescale = 256 , Compare A = Toggle , Clear Timer = 1
Pwm0a = 10


' software UART
Open "comb.1:9600,8,n,1" For Output As #1



Do

  ' setting of speed

  If Pinb.3 = 0 Then
     Dithalf = 33                                           ' duration of half a dit in ms; speed is 90 characters / min.                                      '
     Else
     Dithalf = 50                                           ' duration of half a dit in ms; speed is 60 characters / min.
  End If

  Tpconst = Dithalf * 3                                     ' 1.5 * Dit; maximum tinme for a dit
  Tnpconst = Dithalf * 4                                    ' 2 * Dit; minimum time for a dah
  Iconst = Dithalf * 6                                      ' space between words


  Charactercode = 1

  Do
     Getkeytimes

     If Idling > Iconst And Pinb.4 = 0 Then                 ' inserts space between morse words
        Gosub Printdisplay                                  ' may be deactivated if you want to insert spaces only by <AS>
     End If

     Shift Charactercode , Left , 1
     If Timepressed > Tpconst Then
         Incr Charactercode
     End If
  Loop Until Timenotpressed > Tnpconst
  Gosub Printdisplay

Loop




Sub Printdisplay
  Restore Morsetable
  C = Lookup(charactercode , Morsetable)

  If C > 64 Then                                            ' may be deactivated if you want capital letters
      C = C + 32                                            ' transforms capital letters into small letters
  End If

  Put #1 , C                                                ' ASCII output via software UART
End Sub




Sub Getkey
  Key = Not Pinb.2                                          ' Key is attached to PB2, if key pressed -> low
  If Key > 0 Then
    Ddrb.0 = 1
  Else
    Ddrb.0 = 0
  End If
End Sub




Sub Getkeytimes
  Timepressed = 0
  Timenotpressed = 0
  Idling = 0
  Do
    Getkey
    Waitms 1
    Idling = Idling + 1
  Loop Until Key > 0

  Do
    Waitms 1
    Timepressed = Timepressed + 1
    Getkey
  Loop Until Key = 0
  Do
    Waitms 1
    Timenotpressed = Timenotpressed + 1
    Getkey
  Loop Until Key > 0 Or Timenotpressed > Tnpconst
End Sub





Morsetable:
Data 0 ,
Data 32,                                                    ' "space"       00000001  ASCII 32
Data 69,                                                    ' "E",      . , 00000010, ASCII 69
Data 84,                                                    ' "T",      - , 00000011, ASCII 84
Data 73,                                                    ' "I",     .. , 00000100, ASCII 73
Data 65,                                                    ' "A",     .- , 00000110, ASCII 65
Data 78,                                                    ' "N",     -. , 00000110, ASCII 78
Data 77,                                                    ' "M",     -- , 00000111, ASCII 77
Data 83,                                                    ' "S",    ... , 00001000, ASCII 83
Data 85,                                                    ' "U",    ..- , 00001001, ASCII 85
Data 82,                                                    ' "R",    .-. , 00001010, ASCII 82
Data 87,                                                    ' "W",    .-- , 00001011, ASCII 87
Data 68,                                                    ' "D",    -.. , 00001100, ASCII 68
Data 75,                                                    ' "K",    -.- , 00001101, ASCII 75
Data 71,                                                    ' "G",    --. , 00001110, ASCII 71
Data 79,                                                    ' "O",    --- , 00001111, ASCII 79
Data 72,                                                    ' "H",   .... , 00010000, ASCII 72
Data 86,                                                    ' "V",   ...- , 00010001, ASCII 86
Data 70,                                                    ' "F",   ..-. , 00010010, ASCII 70
Data 0,
Data 76,                                                    ' "L",   .-.. , 00010100, ASCII 76
Data 0 ,                                                    '
Data 80,                                                    ' "P",   .--. , 00010110, ASCII 80
Data 74,                                                    ' "J",   .--- , 00010111, ASCII 74
Data 66,                                                    ' "B",   -... , 00011000, ASCII 66
Data 88,                                                    ' "X",   -..- , 00011001, ASCII 88
Data 67,                                                    ' "C",   -.-. , 00011010, ASCII 67
Data 89,                                                    ' "Y",   -.-- , 00011011, ASCII 89
Data 90,                                                    ' "Z",   --.. , 00011100, ASCII 90
Data 81,                                                    ' "Q",   --.- , 00011101, ASCII 81
Data 0,
Data 0,
Data 53,                                                    ' "5",  ..... , 00100000, ASCII 53
Data 52,                                                    ' "4",  ....- , 00100001, ASCII 52
Data 0,
Data 51,                                                    ' "3",  ...-- , 00100011, ASCII 51
Data 0,
Data 0,
Data 0,
Data 50,                                                    ' "2",  ..--- , 00100111, ASCII 50
Data 32,                                                    ' "as"  .-... , 00101000, ASCII 32
Data 0,
Data 43,                                                    ' "+"   .-.-. , 00101010  ASCII 43
Data 0,
Data 0,
Data 0,
Data 0,
Data 49,                                                    ' "1",  .---- , 00101111, ASCII 49
Data 54,                                                    ' "6",  -.... , 00110000, ASCII 54
Data 61,                                                    ' "="   -...- , 00110001, ASCII 61
Data 47,                                                    ' "/"   -..-. , 00110010, ASCII 47
Data 0,
Data 0,
Data 0,
Data 13,                                                    ' "kn"  -.--. , 00110110  ASCII 13 (Return)
Data 0,
Data 55,                                                    ' "7",  --... , 00111000, ASCII 55
Data 0,
Data 0,
Data 0,
Data 56,                                                    ' "8",  ---.. , 00111100, ASCII 56
Data 0,
Data 57,                                                    ' "9",  ----. , 00111110, ASCII 57
Data 48,                                                    ' "0",  ----- , 00111111, ASCII 48
Data 8,                                                     ' "......"    , 01000000, ASCII 8 (Backspace)
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 63,                                                    ' "?", ..--.. , 01001100, ASCII 63
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 34 ,                                                   ' "'"  .-..-. , 01010010, ASCII 34
Data 0,
Data 0,
Data 46,                                                    ' ".", .-.-.- , 01010101, ASCII 46
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 45,                                                    ' "-", -....- , 01100001, ASCII 45
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 41,                                                    ' ")", -.--.- , 01101101, ASCII 41
Data 0,
Data 0,
Data 0,
Data 0,
Data 0,
Data 44,                                                    ' ",", --..-- , 01110011, ASCII 44
Data 0,
Data 0,
Data 0,
Data 0,
Data 58,                                                    ' ":", ---... , 01111000, ASCII 58
Data 0,
Data 0,
Data 0,
Data 0,
Data 0
Data 0,
Data 0,
Data 8,                                                     ' "......."  , 10000000, ASCII 8 (Backspace)